==> untahris-2.0-src.tgz: untahris/thrust.cpp <==
// Untahris (Common Playground)
// copyright (C) 2006 longuens@users.sourceforge.net
// Released under GNU General Public License (see the file COPYING)
// --------------------------------------------------------------
// Thruster.

struct OThruster : TPlayObject {
  public:
  int x;
  int y;
  int dx;
  int dy;
  int dir;
  OThruster(int _owner, int _x, int _y) : 
    x(Ctr(_x)), y(Ctr(_y)) {
    owner = _owner;
    time = g.time;
    dx = 0; dy = 0; dir = 0;
    g.objects.push_back(this);
    }
  
  OThruster(TBuffer &b) {
    b.get(owner); 
    b.get(x); b.get(y);
    b.get(dx); b.get(dy);
    b.get(time); b.get(dir);
    }
  void save(TBuffer &b) {
    b.putChar(3);
    b.put(owner); 
    b.put(x); b.put(y);
    b.put(dx); b.put(dy);
    b.put(time); b.put(dir);
    }

  int oo;
  
  bool testCrash(int x, int y) {
    int nx = DeCtr(x);
    int ny = DeCtr(y);
    if(nx<0) nx = 0;
    if(ny<0) ny = 0;
    if(nx>g.sizeX-1) nx = g.sizeX-1;
    if(ny>g.sizeY-1) ny = g.sizeY-1;
    CellType m = bigType(nx, ny);
    oo = g.owner[ny][nx];
    if(m != cFree) {
      switch(m) {
        case cBonusLevel:
          d.playSoundAt(d.sndBlue, nx, ny);
          ow()->level += 10;
          ow()->score += scoreGain() * 10;
          break;
        case cBonusScore: 
          d.playSoundAt(d.sndGreen, nx, ny);
          ow()->typeval += 1000;
          ow()->score += scoreGain();
          break;
        case cBonusAmmo:
          d.playSoundAt(d.sndRed, nx, ny);
          ow()->ammo += 10;
          ow()->score += scoreGain();
          break;
        }
      placeBigRandomly(m, 0);
      clearBig(nx, ny, cFree, 0);
      }
    return destroyAt(nx, ny, owner, dfDestroyCannons);
    }
    
  void go() {
    x += dx;
    y += dy;
    dy += 6;
    if(g.pl[owner]->key[keyRight]) dir += 15;
    if(g.pl[owner]->key[keyLeft]) dir += 345;
    dir %= 360;
    if(g.pl[owner]->key[keyUp]) {
      dx += degx[dir]/2;
      dy += degy[dir]/2;
      ow()->typeval -= 20;
      }
    bool b = false;
    int oo2 = ow()->index;
    if(testCrash(x, y)) b = true, oo2 = oo;
    if(testCrash(x+8*degx[(dir+120)%360], y+8*degy[(dir+120)%360])) b = true, oo2 = oo;
    if(testCrash(x+8*degx[(dir+240)%360], y+8*degy[(dir+240)%360])) b = true, oo2 = oo;
    if(testCrash(x+8*degx[dir], y+8*degy[dir])) b = true, oo2 = oo;
    for(int o=0; o<size(g.objects); o++) if(g.objects[o])
      if(g.objects[o]->reactToThruster(this)) return;
    if(b || ow()->typeval < 0) {
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      killedBy(oo2);
      return;
      }
    else time += spd();
    }
  void draw() {
    DIc(icThrust).drawAtRotated(CTG(x), CTG(y), degx[dir], degy[dir]);
    }
  void reactToKey(PlayerKey pk, bool pressed) {
    if(!pressed) return;
    if(pk == keyFire && ow()->useAmmo(2)) {
      d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
      new OBullet(owner, x + degx[dir] * 8, y + degy[dir] * 8, 
        dx/spd() + (degx[dir] * bulletSpeed()) / 128, dy/spd() + (degy[dir] * bulletSpeed()) / 128);
      }
    if(pk == keyEarth && ow()->useAmmo(2)) {
      d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
      new OBullet(owner, x - degx[dir] * 8, y - degy[dir] * 8, 
        dx/spd() - (degx[dir] * bulletSpeed()) / 128, dy/spd() - (degy[dir] * bulletSpeed()) / 128);
      }
    if(pk == keyAir && ow()->useAmmo(4)) {
      d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
      new OBullet(owner, x + degy[dir] * 8, y - degx[dir] * 8, 
        dx/spd() + (degy[dir] * bulletSpeed()) / 128, dy/spd() - (degx[dir] * bulletSpeed()) / 128);
      new OBullet(owner, x - degy[dir] * 8, y + degx[dir] * 8, 
        dx/spd() - (degy[dir] * bulletSpeed()) / 128, dy/spd() + (degx[dir] * bulletSpeed()) / 128);
      }
    }
  bool reactToThruster(OThruster *enemy) {
    if(this == enemy) return false;
    if(calcDistance(x-enemy->x, y-enemy->y) < 8*128*2) {
      int o0 = owner;
      int oe = enemy->owner;
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      enemy->killedBy(o0);
      killedBy(oe);
      return true;
      }
    return false;
    }
  bool reactToBullet(OBullet *bullet) {
    if(calcDistance(x-bullet->x, y-bullet->y) < 8*128) {
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      killedBy(bullet->owner);
      return true;
      }
    return false;
    }
  bool catchesRope(int rx, int ry) {
    if(calcDistance(x-rx, y-ry) < 8*128) {
      return true;
      }
    return false;
    }
  void pullRope(int& rx, int& ry, int &drx, int &dry, int ddx, int ddy) {
    rx = x; ry = y; drx = dx; dry = dy;
    dx -= ddx/3; dy -= ddy/3;
    }
  };

==> untahris-3.1-src.tgz: untahris/thrust.cpp <==
// Untahris (Common Playground)
// copyright (C) 2006 longuens@users.sourceforge.net
// Released under GNU General Public License (see the file COPYING)
// --------------------------------------------------------------
// Thruster.

struct OThruster : TPlayObject {
  public:
  int32 x;
  int32 y;
  int32 dx;
  int32 dy;
  int32 dir;
  OThruster(int _owner, int _x, int _y) : 
    x(Ctr(_x)), y(Ctr(_y)) {
    owner = _owner;
    time = g.time;
    dx = 0; dy = 0; dir = 0;
    g.objects.push_back(this);
    }
  
  OThruster(TBuffer &b) {
    b.get(owner); 
    b.get(x); b.get(y);
    b.get(dx); b.get(dy);
    b.get(time); b.get(dir);
    }
  void save(TBuffer &b) {
    b.putChar(3);
    b.put(owner); 
    b.put(x); b.put(y);
    b.put(dx); b.put(dy);
    b.put(time); b.put(dir);
    }

  int32 oo;
  
  bool testCrash(int x, int y) {
    int nx = DeCtr(x);
    int ny = DeCtr(y);
    if(nx<0) nx = 0;
    if(ny<0) ny = 0;
    if(nx>g.sizeX-1) nx = g.sizeX-1;
    if(ny>g.sizeY-1) ny = g.sizeY-1;
    CellType m = bigType(nx, ny);
    oo = g.owner[ny][nx];
    if(m != cFree && m != cCannonA && m != cCannonF) {
      switch(m) {
        case cBonusLevel:
          d.playSoundAt(d.sndBlue, nx, ny);
          ow()->level ++;
          ow()->score += scoreGain() * 10;
          break;
        case cBonusScore: 
          d.playSoundAt(d.sndGreen, nx, ny);
          ow()->typeval += gc.bonusFuel;
          ow()->score += scoreGain();
          break;
        case cBonusAmmo:
          d.playSoundAt(d.sndRed, nx, ny);
          ow()->gainAmmo();
          ow()->score += scoreGain();
          break;
        }
      if(gc.regenerate(m)) 
        placeBigRandomly(m, 0);
      clearBig(nx, ny, cFree, 0);
      }
    return destroyAt(nx, ny, owner, dfDestroyCannons | dfThrust);
    }
    
  void go() {
    x += dx;
    y += dy;
    dy += 6;
    if(g.pl[owner]->key[keyRight]) dir += 15;
    if(g.pl[owner]->key[keyLeft]) dir += 345;
    dir %= 360;
    if(g.pl[owner]->key[keyUp]) {
      dx += (degx[dir] * gc.thrustAcc) / 100;
      dy += (degy[dir] * gc.thrustAcc) / 100;
      ow()->typeval -= gc.fuelUseSpeed;
      }
    bool b = false;
    int oo2 = ow()->index;
    if(testCrash(x, y)) b = true, oo2 = oo;
    if(testCrash(x+8*degx[(dir+120)%360], y+8*degy[(dir+120)%360])) b = true, oo2 = oo;
    if(testCrash(x+8*degx[(dir+240)%360], y+8*degy[(dir+240)%360])) b = true, oo2 = oo;
    if(testCrash(x+8*degx[dir], y+8*degy[dir])) b = true, oo2 = oo;
    for(int o=0; o<size(g.objects); o++) if(g.objects[o])
      if(g.objects[o]->reactToThruster(this)) return;
    if(b || ow()->typeval < 0) {
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      killedBy(oo2);
      return;
      }
    else time += spd();
    }
  void draw() {
    DIc(icThrust).drawAtRotated(CTG(x), CTG(y), degx[dir], degy[dir]);
    }
  void reactToKey(PlayerKey pk, bool pressed) {
    if(!pressed) return;
    if(pk == keyFire && ow()->useAmmo()) {
      d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
      new OBullet(owner, x + degx[dir] * 8, y + degy[dir] * 8, 
        dx/spd() + (degx[dir] * bulletSpeed()) / 128, dy/spd() + (degy[dir] * bulletSpeed()) / 128);
      }
    if(pk == keyEarth && ow()->useAmmo()) {
      d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
      new OBullet(owner, x - degx[dir] * 8, y - degy[dir] * 8, 
        dx/spd() - (degx[dir] * bulletSpeed()) / 128, dy/spd() - (degy[dir] * bulletSpeed()) / 128);
      }
    if(pk == keyAir && ow()->useAmmo(ec().ammoPerShoot * 2)) {
      d.playSoundAt(d.sndShoot, DeCtr(x), DeCtr(y));
      new OBullet(owner, x + degy[dir] * 8, y - degx[dir] * 8, 
        dx/spd() + (degy[dir] * bulletSpeed()) / 128, dy/spd() - (degx[dir] * bulletSpeed()) / 128);
      new OBullet(owner, x - degy[dir] * 8, y + degx[dir] * 8, 
        dx/spd() - (degy[dir] * bulletSpeed()) / 128, dy/spd() + (degx[dir] * bulletSpeed()) / 128);
      }
    }
  bool reactToThruster(OThruster *enemy) {
    if(this == enemy) return false;
    if(calcDistance(x-enemy->x, y-enemy->y) < 8*128*2) {
      int o0 = owner;
      int oe = enemy->owner;
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      enemy->killedBy(o0);
      killedBy(oe);
      return true;
      }
    return false;
    }
  bool reactToBullet(OBullet *bullet) {
    if(calcDistance(x-bullet->x, y-bullet->y) < 8*128) {
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      killedBy(bullet->owner);
      return true;
      }
    return false;
    }
  void reactToCannon(OCannonball *ball) {
    if(calcDistance(x-ball->tx, y-ball->ty) < 8*128 + 512) {
      d.playSoundAt(d.sndDeath, DeCtr(x), DeCtr(y));
      killedBy(ball->owner);
      }
    }
  bool catchesRope(int rx, int ry) {
    if(calcDistance(x-rx, y-ry) < 8*128) {
      return true;
      }
    return false;
    }
  void pullRope(OWorm *worm, int ddx, int ddy); 
  };
